var platformModule = require("platform");
var frameModule = require("ui/frame");
var view = require("ui/core/view");
var AbsoluteLayout = require("ui/layouts/absolute-layout").AbsoluteLayout;
var StackLayout = require("ui/layouts/stack-layout").StackLayout;
var GridLayout = require("ui/layouts/grid-layout").GridLayout;
var ScrollView = require("ui/scroll-view").ScrollView;
var Repeater = require("ui/repeater").Repeater;
var Label = require("ui/label").Label;
var helpers = require("./helpers");

var NASlideShow = (function(_super) {
  __extends(NASlideShow, _super);
  function NASlideShow() {
    var _this = this;
    _super.apply(this, arguments);
    
    var _slideIndex = 0, _getMeasuredSize, _itemsPush, _slideLength = [];
    _this._loaded = false;
    
    // FUNCTION - INIT SLIDESHOW
    _this.init = function(template, props = {}) {
      if(!_this._loaded) {
        var defaults = {
          items: false,
          itemTemplate: "<GridLayout />",
          enableBounce: false,
          fallbackWidth: platformModule.screen.mainScreen.widthDIPs,
          fallbackHeight: platformModule.screen.mainScreen.heightDIPs,
          clipToBounds: true,
          indicators: {
            size: 10,
            color: "#808080",
            activeColor: false,
            borderWidth: 0,
            borderColor: "#404040",
            horizontalAlignment: "center",
            verticalAlignment: "bottom",
            iconShadow: false,
//            blur: false, // Experimental (not working currently)
            forceVisibility: false,
            disabled: false
          }
        };
        for(var key in defaults) if(!props.hasOwnProperty(key)) props[key] = defaults[key];
        for(var key in defaults.indicators) if(!props.indicators.hasOwnProperty(key)) props.indicators[key] = defaults.indicators[key];
        for(var key in defaults.indicators.iconShadow) if(!props.indicators.iconShadow.hasOwnProperty(key)) props.indicators.iconShadow[key] = defaults.indicators.iconShadow[key];

        // THIS
        _this._loaded = true;
        _this.view = template;
        _this.viewsList = {};
        _this.props = props;
        _getMeasuredSize = function(xy) {
          if(xy === "width") return template.getMeasuredWidth() || _this.props.fallbackWidth;
            else if(xy === "height") return template.getMeasuredHeight() || _this.props.fallbackHeight;
        };
        
        // XML REPEATER CHECK
        var xmlRepeater = (template.getChildAt(0) && template.getChildAt(0).typeName === "Repeater" ? template.getChildAt(0) : false);
        if(xmlRepeater) props.items = false;
        
        // 1) Create ScrollView etc
        var scrollView = _this.viewsList.scrollView = new ScrollView();
        scrollView.width = _getMeasuredSize("width");
        scrollView.height = _getMeasuredSize("height");
        if(frameModule.topmost().ios) scrollView.ios.clipsToBounds = props.clipToBounds;

        // 2) Create Repeater / StackLayout (container)
        if(props.items) {
          var repeater = _this.viewsList.repeater = new Repeater();
          repeater.itemTemplate = props.itemTemplate;
          var items = _this.items = repeater.items = props.items;
          var container = _this.viewsList.container = repeater.itemsLayout = new StackLayout();
        } else {
          if(xmlRepeater) {
            var repeater = _this.viewsList.repeater = xmlRepeater;
            var items = _this.items = xmlRepeater.items;
            var container = _this.viewsList.container = xmlRepeater.itemsLayout = new StackLayout();
          } else {
            var container = _this.viewsList.container = scrollView.content = new StackLayout();
            var slides = [];
            template._eachChildView(function(view) { slides.push(view); });
            template.removeChildren();
          }
        }

        // 3) Set classes and other properties
        template.className = "NASlideShow";
        scrollView.orientation = "horizontal";
        if(props.items || xmlRepeater) repeater.className = "NASlideShow-repeater";
        container.className = "NASlideShow-container";
        container.orientation = "horizontal";

        // 4) Insert views
        if(props.items) {
          scrollView.content = repeater;
        } else {
          if(xmlRepeater) {
            xmlRepeater.parent.removeChild(xmlRepeater);
            scrollView.content = xmlRepeater;
            props.items = _this.props.items = xmlRepeater.items;
          } else {
            slides.forEach(function(view, index) {
              container.insertChild(view, index);
            });
            scrollView.content = container;
          }
        }
        template.insertChild(scrollView, 0);

        // 5) Create indicators view
        var indicators_container = new GridLayout();
        indicators_container.width = _getMeasuredSize("width");
        indicators_container.height = props.indicators.size + 10;
        indicators_container.isUserInteractionEnabled = false;
        if(props.indicators.disabled || (!props.indicators.forceVisibility && _this.getSlidesCount() <= 1)) indicators_container.visibility = "collapsed";
        indicators_container.translateY = (props.indicators.verticalAlignment === "top" ? 15 : -15);
        if(props.indicators.horizontalAlignment === "left") indicators_container.translateX = 10;
          else if(props.indicators.horizontalAlignment === "right") indicators_container.translateX = -10;
        
        var indicators = _this.viewsList.indicators = new StackLayout();
        indicators.horizontalAlignment = props.indicators.horizontalAlignment;
        indicators.className = "NASlideShow-indicators";
        indicators.orientation = "horizontal";

        setTimeout(function() {
          AbsoluteLayout.setTop(indicators_container, (props.indicators.verticalAlignment === "top" ? 0 : _getMeasuredSize("height") - indicators_container.height));
        }, 0);

        indicators_container.insertChild(indicators, 0);
        template.insertChild(indicators_container, 1);
        
        var appendIndicator = function(i) {
          var view = container.getChildAt(i);
          if(view) view.width = _getMeasuredSize("width");
          _slideLength.push(_getMeasuredSize("width") * i);
          
          var dot = new StackLayout();
          dot.className = "NASlideShow-indicators-dot";
          dot.width = dot.height = props.indicators.size;
          dot.margin = 5;
//            dot.backgroundColor = helpers.color(props.indicators.color, (i === 0 ? 0.75 : 0.25));
          dot.backgroundColor = props.indicators.color;
          if(props.indicators.activeColor && i === _this.getSlideIndex()) dot.backgroundColor = props.indicators.activeColor;
          dot.borderRadius = props.indicators.size / 2;
          dot.horizontalAlignment = _this.props.indicators.horizontalAlign || "center";
          dot.clipToBounds = false;
          dot.opacity = (i === 0 ? 0.75 : 0.25);

          if(props.indicators.borderWidth) { // Border
            dot.borderWidth = props.indicators.borderWidth;
            dot.borderColor = props.indicators.borderColor;
          }

//            if(props.indicators.blur) { // Blur
//              if(frameModule.topmost().ios) { // iOS
//                var blurView = UIVisualEffectView.alloc().initWithEffect(UIBlurEffect.effectWithStyle(UIBlurEffectStyleLight));
//                blurView.frame = { origin: { x: 0, y: 0 }, size: { width: dot.ios.frame.size.width, height: dot.ios.frame.size.height } };
//                blurView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
//                dot.ios.addSubview(blurView);
//                dot.ios.sendSubviewToBack(blurView);
//              }
//            }

          indicators.insertChild(dot, i);
        };
        
        if(props.indicators.forceVisibility) appendIndicator(0);
        for(var i = 0; i < _this.getSlidesCount(); i++) {
          if((props.indicators.forceVisibility && i > 0) || !props.indicators.forceVisibility) appendIndicator(i);
        }
        
        var scrollX = 0, direction = null;
        var currentSlideIndex = 0, previousSlideIndex = null;
        var incomingDot = null;
        
        scrollView.on("scroll", function(args) {
          direction = (scrollX > args.scrollX ? "left" : (scrollX === args.scrollX ? null : "right"));
          
          if(direction === "left") incomingDot = indicators.getChildAt(Math.max(currentSlideIndex, 1) - 1);
            else if(direction === "right") incomingDot = indicators.getChildAt(Math.min(currentSlideIndex, 1) + 1);
            else incomingDot = indicators.getChildAt(currentSlideIndex);
          
          // When scroll has finished
          if(_slideLength.indexOf(args.scrollX) >= 0) {
            _slideLength.forEach(function(value, index, array) {
              if(args.scrollX === value) {
                previousSlideIndex = currentSlideIndex;
                currentSlideIndex = index;
              }
              
              if(index === array.length-1) { // Last
                if(previousSlideIndex !== currentSlideIndex) {
                  var i = 0;
                  indicators._eachChildView(function(indicator) {
                    var animationProps = { duration: 100 };
                    
                    if(i === currentSlideIndex) {
                      animationProps.opacity = 0.75;
                      
                      if(props.indicators.activeColor) {
                        if(indicator.className !== "NASlideShow-indicators-icon") { // Dot
                          animationProps.backgroundColor = props.indicators.activeColor;
                        } else { // Icon
                          helpers.nativeAnimation(indicator, { duration: 100 }, function() {
                            indicator.getChildAt(0).color = props.indicators.activeColor;
                          });
                        }
                      }
                    } else {
                      animationProps.opacity = 0.25;
                      
                      if(props.indicators.activeColor) {
                        if(indicator.className !== "NASlideShow-indicators-icon") { // Dot
                          animationProps.backgroundColor = props.indicators.color;
                        } else { // Icon
                          helpers.nativeAnimation(indicator, { duration: 100 }, function() {
                            indicator.getChildAt(0).color = props.indicators.color;
                          });
                        }
                      }
                    }
                    
                    indicator.animate(animationProps);
                    i++;
                  });
                  
                  _slideIndex = currentSlideIndex;
                  
                  // CREATE EVENT LISTENER
                  var eventData = {
                    eventName: "slideChange",
                    object: _this,
                    index: currentSlideIndex,
                    length: _this.getSlidesCount()
                  };
                  _this.notify(eventData);
                }
              }
            });
          }
          
          scrollX = args.scrollX;
        });
        
        if(frameModule.topmost().ios) { // iOS
          var nativeScrollView = scrollView.ios;
          
          nativeScrollView.pagingEnabled = true;
          if(!props.enableBounce) nativeScrollView.bounces = false;
          nativeScrollView.showsHorizontalScrollIndicator = false;
          nativeScrollView.showsVerticalScrollIndicator = false;
        } else if(frameModule.topmost().android) { // Android
          
        }
        
        // REPEATER ITEMS PROPERTY CHANGES
        if(_this.items) {
          _this.items.on("propertyChange", function(e) {
            if(!_itemsPush) insertSlide();
          });
        }
      }
    };
    
    
    // FUNCTION - INSERT SLIDE
    _this.insertSlide = function(slideView, props = {}) {
      if(_this.items) { // Repeater
        if(!_itemsPush) {
          _itemsPush = true;
          _this.items.splice((typeof props.atIndex !== "undefined" ? props.atIndex : _this.getSlidesCount()), 0, (slideView || null));
          
          insertSlide(slideView, props);
        }
      } else { // No Repeater
        insertSlide(slideView, props);
      }
    };
    
    function insertSlide(slideView, props = {}) {
      var defaults = {
        atIndex: _this.getSlidesCount(),
        indicator: {
          icon: false,
          fontFamily: false,
          float: false
        }
      };
      for(var key in defaults) if(!props.hasOwnProperty(key)) props[key] = defaults[key];
      for(var key in defaults.indicator) if(!props.indicator.hasOwnProperty(key)) props.indicator[key] = defaults.indicator[key];
      
      _itemsPush = undefined;
      if(_this.items && slideView && (slideView.ios || slideView.android)) return false;
      if(props.indicator.float && props.atIndex !== _this.getSlidesCount() || _this.props.indicators.horizontalAlignment === "right") props.indicator.float = false;
      _this.viewsList.indicators.translateX = 0;
      
      // INDICATOR
      var indicator = new StackLayout();
      indicator.className = (props.indicator.icon ? "NASlideShow-indicators-icon" : "NASlideShow-indicators-dot");
      indicator.width = indicator.height = _this.props.indicators.size;
      indicator.margin = 5;
      if(!props.indicator.icon) indicator.backgroundColor = _this.props.indicators.color;
      indicator.borderRadius = _this.props.indicators.size / 2;
      indicator.opacity = 0.25;
      indicator.clipToBounds = false;

      if(_this.props.indicators.borderWidth && !props.indicator.icon) { // Border
        indicator.borderWidth = _this.props.indicators.borderWidth;
        indicator.borderColor = _this.props.indicators.borderColor;
      }

//        if(_this.props.indicators.blur) { // Blur
//          if(frameModule.topmost().ios) { // iOS
//            var blurView = UIVisualEffectView.alloc().initWithEffect(UIBlurEffect.effectWithStyle(UIBlurEffectStyleLight));
//            blurView.frame = { origin: { x: 0, y: 0 }, size: { width: indicator.ios.frame.size.width, height: indicator.ios.frame.size.height } };
//            blurView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
//            indicator.ios.addSubview(blurView);
//            indicator.ios.sendSubviewToBack(blurView);
//          }
//        }

      if(props.indicator.icon) {
        if(props.indicator.float) _this.viewsList.indicators.translateX = _this.props.indicators.size;

        var label = new Label();
        label.text = props.indicator.icon;
        label.color = _this.props.indicators.color;
        label.marginTop = -3;
        label.marginLeft = -1;
        label.fontSize = _this.props.indicators.size * 2;
        if(props.fontFamily) label.fontFamily = props.fontFamily;
        if(_this.props.indicators.iconShadow) helpers.textShadow(label, _this.props.indicators.iconShadow);

        indicator.insertChild(label);
      }

      var indicators = _this.viewsList.indicators;
      indicators.insertChild(indicator, props.atIndex);

      if(props.atIndex === 0) {
        var i = 0;
        _this.viewsList.indicators._eachChildView(function(indicator) {
          indicator.opacity = (i === 0 ? 1 : 0.25);
          i++;
        });
      }
      
      // SLIDE VIEW
      var formula = (_this.getSlidesCount() - (_this.items ? 1 : 0));
      _slideLength.push(_getMeasuredSize("width") * formula);
      
      if(_this.items) { // Repeater
        _this.viewsList.container._eachChildView(function(view) {
          view.width = _getMeasuredSize("width");
        });
      } else { // No Repeater
        slideView.width = _getMeasuredSize("width");
        _this.viewsList.container.insertChild(slideView, props.atIndex);
      }
      
      // CHECK forceVisibility PROPERTY
      if(!_this.props.indicators.disabled && !_this.props.indicators.forceVisibility && _this.getSlidesCount() > 1)
        _this.viewsList.indicators.parent.visibility = "visible";
      
      return _this;
    }
    
    
    // FUNCTION - REMOVE SLIDE
    _this.removeSlide = function(index) {
      var container = _this.viewsList.container;
      var slideView = container.getChildAt(index);
      var indicators = _this.viewsList.indicators;
      var indicator = indicators.getChildAt(index);
      
      _slideLength.pop();
      indicators.removeChild(indicator);
      
      if(_this.items) {
        _itemsPush = true;
        _this.items.splice(index, 1);
        
        container._eachChildView(function(view) {
          view.width = _getMeasuredSize("width");
        });
      } else {
        container.removeChild(slideView);
      }
      
      var indicatorNext = indicators.getChildAt(_this.getSlideIndex()) || indicators.getChildAt(_this.getSlideIndex()-1);
      if(indicatorNext) {
        indicatorNext.opacity = 0.75;
        if(_this.props.indicators.activeColor) {
          if(indicatorNext.getChildAt(0)) indicatorNext.getChildAt(0).color = _this.props.indicators.activeColor;
            else indicatorNext.backgroundColor = _this.props.indicators.activeColor;
        }
      }
      
      _itemsPush = undefined;
      return _this;
    };
    
    
    // FUNCTION - SCROLL TO SLIDE
    _this.scrollToSlide = function(index, animated = false) {
      _this.viewsList.scrollView.scrollToHorizontalOffset(_slideLength[index], animated);
    };
    
    
    // FUNCTION - GET SLIDE
    _this.getSlide = function(index) {
      if(_this.items) return _this.items.getItem(index);
        else return _this.viewsList.container.getChildAt(index);
    };
    
    
    // FUNCTION - SET SLIDE
    _this.setSlide = function(index, value) {
      if(_this.items) { // Repeater
        _this.items.setItem(index, value);
        
        _this.viewsList.container._eachChildView(function(view) {
          view.width = _getMeasuredSize("width");
        });
      } else { // No Repeater
        _this.viewsList.container.getChildAt(index).removeChildren();
        _this.viewsList.container.getChildAt(index).insertChild(value, 0);
      }
    };
    
    
    // FUNCTION - CLEAR SLIDES
    _this.clearSlides = function() {
      if(_this.items) while(_this.items.length) _this.items.pop();
        else _this.viewsList.container.removeChildren();
      
      _this.viewsList.indicators.removeChildren();
    };
    
    
    // FUNCTION - GET SLIDES COUNT
    _this.getSlidesCount = function() {
      if(_this.items) return _this.items.length;
        else return _this.viewsList.container.getChildrenCount();
    };
    
    
    // FUNCTION - GET SLIDE INDEX
    _this.getSlideIndex = function() {
      return _slideIndex;
    };
    
  }
  
  return NASlideShow;
})(view.View);

module.exports = NASlideShow;